class HamlRenderer
  class HamlHelper
    attr_reader :stats

    def initialize(templatedir, stats, verbose, engineConfig)
      @templatedir = templatedir
      @stats = stats
      @verbose = verbose
      @layout = nil
      @engineConfig = engineConfig

      Dir.glob(File.join(templatedir, 'helpers', '*.rb')).sort.each do |file|
        eval(IO::readlines(file).join(''))
      end
    end

    def partial(name, hash = {})
      name = name.to_s
      puts "rendering partial '#{name}' ..." if @verbose
      hash = hash.to_h unless hash.is_a? Hash
      lines = IO::readlines(File.join(@templatedir, 'partials', "#{name}.haml")).join('')
      engine = Haml::Engine.new(lines, @engineConfig)
      engine.render(self, hash)
    end

    def layout(name)
      @layout = name.to_s
    end

    def get_layout
      @layout
    end
  end

  def initialize(templatedir, outdir, verbose)
    @templatedir = templatedir
    @outdir = outdir
    @verbose = verbose
    @engineConfig = {}
  end

  def name
    'haml'
  end

  def handle?(file)
    file =~ /\.haml$/
  end

  def render(file, stats)
    ifile = File.join(@templatedir, file)
    ofile = File.join(@outdir, File.basename(file, '.haml') + '.html')

    lines = IO::readlines(ifile).join('')

    helper = HamlHelper.new(@templatedir, stats, @verbose, @engineConfig)

    engine = Haml::Engine.new(lines, @engineConfig)
    lines = engine.render(helper)

    if !helper.get_layout.nil?
      puts "rendering layout '#{helper.get_layout}' ..." if @verbose
      layout = IO::readlines(File.join(@templatedir, 'layouts', helper.get_layout + '.haml')).join('')
      engine = Haml::Engine.new(layout, @engineConfig)
      lines = engine.render(helper, :content => lines)
    end

    File.new(ofile, 'w').write(lines)
  end
end

Renderer.register HamlRenderer
